@@ -15,8 +15,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import math
 
+import ddt
 import mock
 from oslotest import base as test_base
 import six
@@ -30,6 +32,12 @@ load_tests = testscenarios.load_tests_apply_scenarios
 
 class StrUtilsTest(test_base.BaseTestCase):
 
+    @mock.patch("six.text_type")
+    def test_bool_bool_from_string_no_text(self, mock_text):
+        self.assertTrue(strutils.bool_from_string(True))
+        self.assertFalse(strutils.bool_from_string(False))
+        self.assertEqual(0, mock_text.call_count)
+
     def test_bool_bool_from_string(self):
         self.assertTrue(strutils.bool_from_string(True))
         self.assertFalse(strutils.bool_from_string(False))
@@ -143,6 +151,21 @@ class StrUtilsTest(test_base.BaseTestCase):
         self.assertEqual(1, strutils.int_from_bool_as_string(True))
         self.assertEqual(0, strutils.int_from_bool_as_string(False))
 
+    def test_is_valid_boolstr(self):
+        self.assertTrue(strutils.is_valid_boolstr('true'))
+        self.assertTrue(strutils.is_valid_boolstr('false'))
+        self.assertTrue(strutils.is_valid_boolstr('yes'))
+        self.assertTrue(strutils.is_valid_boolstr('no'))
+        self.assertTrue(strutils.is_valid_boolstr('y'))
+        self.assertTrue(strutils.is_valid_boolstr('n'))
+        self.assertTrue(strutils.is_valid_boolstr('1'))
+        self.assertTrue(strutils.is_valid_boolstr('0'))
+        self.assertTrue(strutils.is_valid_boolstr(1))
+        self.assertTrue(strutils.is_valid_boolstr(0))
+
+        self.assertFalse(strutils.is_valid_boolstr('maybe'))
+        self.assertFalse(strutils.is_valid_boolstr('only on tuesdays'))
+
     def test_slugify(self):
         to_slug = strutils.to_slug
         self.assertRaises(TypeError, to_slug, True)
@@ -164,6 +187,7 @@ class StringToBytesTest(test_base.BaseTestCase):
     _unit_system = [
         ('si', dict(unit_system='SI')),
         ('iec', dict(unit_system='IEC')),
+        ('mixed', dict(unit_system='mixed')),
         ('invalid_unit_system', dict(unit_system='KKK', assert_error=True)),
     ]
 
@@ -236,6 +260,14 @@ class StringToBytesTest(test_base.BaseTestCase):
                     res = getattr(units, unit_prefix)
                 else:
                     res = getattr(units, '%si' % unit_prefix)
+            elif unit_system == 'mixed':
+                # Note: this will return 'i' units as power-of-two,
+                # and other units as power-of-ten.  Additionally, for
+                # compatability a "K" is interpreted as "k" in mixed
+                # mode
+                if unit_prefix == 'K':
+                    unit_prefix = 'k'
+                res = getattr(units, unit_prefix)
             return res
 
         text = ''.join([self.sign, self.magnitude, self.unit_prefix,
@@ -265,43 +297,43 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
 
     def test_json(self):
         # Test 'adminPass' w/o spaces
-        payload = """{'adminPass':'mypassword'}"""
+        payload = """{'adminPass':'TL0EfN33'}"""
         expected = """{'adminPass':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'adminPass' with spaces
-        payload = """{ 'adminPass' : 'mypassword' }"""
+        payload = """{ 'adminPass' : 'TL0EfN33' }"""
         expected = """{ 'adminPass' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' w/o spaces
-        payload = """{'admin_pass':'mypassword'}"""
+        payload = """{'admin_pass':'TL0EfN33'}"""
         expected = """{'admin_pass':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' with spaces
-        payload = """{ 'admin_pass' : 'mypassword' }"""
+        payload = """{ 'admin_pass' : 'TL0EfN33' }"""
         expected = """{ 'admin_pass' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' w/o spaces
-        payload = """{'admin_password':'mypassword'}"""
+        payload = """{'admin_password':'TL0EfN33'}"""
         expected = """{'admin_password':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' with spaces
-        payload = """{ 'admin_password' : 'mypassword' }"""
+        payload = """{ 'admin_password' : 'TL0EfN33' }"""
         expected = """{ 'admin_password' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' w/o spaces
-        payload = """{'password':'mypassword'}"""
+        payload = """{'password':'TL0EfN33'}"""
         expected = """{'password':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' with spaces
-        payload = """{ 'password' : 'mypassword' }"""
+        payload = """{ 'password' : 'TL0EfN33' }"""
         expected = """{ 'password' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'auth_password' w/o spaces
-        payload = """{'auth_password':'mypassword'}"""
+        payload = """{'auth_password':'TL0EfN33'}"""
         expected = """{'auth_password':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'auth_password' with spaces
-        payload = """{ 'auth_password' : 'mypassword' }"""
+        payload = """{ 'auth_password' : 'TL0EfN33' }"""
         expected = """{ 'auth_password' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'secret_uuid' w/o spaces
@@ -312,96 +344,104 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
         payload = """{ 'secret_uuid' : 'myuuid' }"""
         expected = """{ 'secret_uuid' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'token' w/o spaces
+        payload = """{'token':'token'}"""
+        expected = """{'token':'***'}"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'token' with spaces
+        payload = """{ 'token' : 'token' }"""
+        expected = """{ 'token' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
 
     def test_xml(self):
         # Test 'adminPass' w/o spaces
-        payload = """<adminPass>mypassword</adminPass>"""
+        payload = """<adminPass>TL0EfN33</adminPass>"""
         expected = """<adminPass>***</adminPass>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'adminPass' with spaces
         payload = """<adminPass>
-                        mypassword
+                        TL0EfN33
                      </adminPass>"""
         expected = """<adminPass>***</adminPass>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' w/o spaces
-        payload = """<admin_pass>mypassword</admin_pass>"""
+        payload = """<admin_pass>TL0EfN33</admin_pass>"""
         expected = """<admin_pass>***</admin_pass>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' with spaces
         payload = """<admin_pass>
-                        mypassword
+                        TL0EfN33
                      </admin_pass>"""
         expected = """<admin_pass>***</admin_pass>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' w/o spaces
-        payload = """<admin_password>mypassword</admin_password>"""
+        payload = """<admin_password>TL0EfN33</admin_password>"""
         expected = """<admin_password>***</admin_password>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' with spaces
         payload = """<admin_password>
-                        mypassword
+                        TL0EfN33
                      </admin_password>"""
         expected = """<admin_password>***</admin_password>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' w/o spaces
-        payload = """<password>mypassword</password>"""
+        payload = """<password>TL0EfN33</password>"""
         expected = """<password>***</password>"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' with spaces
         payload = """<password>
-                        mypassword
+                        TL0EfN33
                      </password>"""
         expected = """<password>***</password>"""
         self.assertEqual(expected, strutils.mask_password(payload))
 
     def test_xml_attribute(self):
         # Test 'adminPass' w/o spaces
-        payload = """adminPass='mypassword'"""
+        payload = """adminPass='TL0EfN33'"""
         expected = """adminPass='***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'adminPass' with spaces
-        payload = """adminPass = 'mypassword'"""
+        payload = """adminPass = 'TL0EfN33'"""
         expected = """adminPass = '***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'adminPass' with double quotes
-        payload = """adminPass = "mypassword\""""
+        payload = """adminPass = "TL0EfN33\""""
         expected = """adminPass = "***\""""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' w/o spaces
-        payload = """admin_pass='mypassword'"""
+        payload = """admin_pass='TL0EfN33'"""
         expected = """admin_pass='***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' with spaces
-        payload = """admin_pass = 'mypassword'"""
+        payload = """admin_pass = 'TL0EfN33'"""
         expected = """admin_pass = '***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_pass' with double quotes
-        payload = """admin_pass = "mypassword\""""
+        payload = """admin_pass = "TL0EfN33\""""
         expected = """admin_pass = "***\""""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' w/o spaces
-        payload = """admin_password='mypassword'"""
+        payload = """admin_password='TL0EfN33'"""
         expected = """admin_password='***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' with spaces
-        payload = """admin_password = 'mypassword'"""
+        payload = """admin_password = 'TL0EfN33'"""
         expected = """admin_password = '***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'admin_password' with double quotes
-        payload = """admin_password = "mypassword\""""
+        payload = """admin_password = "TL0EfN33\""""
         expected = """admin_password = "***\""""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' w/o spaces
-        payload = """password='mypassword'"""
+        payload = """password='TL0EfN33'"""
         expected = """password='***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' with spaces
-        payload = """password = 'mypassword'"""
+        payload = """password = 'TL0EfN33'"""
         expected = """password = '***'"""
         self.assertEqual(expected, strutils.mask_password(payload))
         # Test 'password' with double quotes
-        payload = """password = "mypassword\""""
+        payload = """password = "TL0EfN33\""""
         expected = """password = "***\""""
         self.assertEqual(expected, strutils.mask_password(payload))
 
@@ -526,40 +566,45 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
         expected = 'test = "param1" : "value"'
         self.assertEqual(expected, strutils.mask_password(payload))
 
-        payload = """{'adminPass':'mypassword'}"""
+        payload = """{'adminPass':'TL0EfN33'}"""
         payload = six.text_type(payload)
         expected = """{'adminPass':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
 
-        payload = ("test = 'node.session.auth.password','-v','mypassword',"
+        payload = """{'token':'mytoken'}"""
+        payload = six.text_type(payload)
+        expected = """{'token':'***'}"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+
+        payload = ("test = 'node.session.auth.password','-v','TL0EfN33',"
                    "'nomask'")
         expected = ("test = 'node.session.auth.password','-v','***',"
                     "'nomask'")
         self.assertEqual(expected, strutils.mask_password(payload))
 
         payload = ("test = 'node.session.auth.password', '--password', "
-                   "'mypassword', 'nomask'")
+                   "'TL0EfN33', 'nomask'")
         expected = ("test = 'node.session.auth.password', '--password', "
                     "'***', 'nomask'")
         self.assertEqual(expected, strutils.mask_password(payload))
 
         payload = ("test = 'node.session.auth.password', '--password', "
-                   "'mypassword'")
+                   "'TL0EfN33'")
         expected = ("test = 'node.session.auth.password', '--password', "
                     "'***'")
         self.assertEqual(expected, strutils.mask_password(payload))
 
-        payload = "test = node.session.auth.password -v mypassword nomask"
+        payload = "test = node.session.auth.password -v TL0EfN33 nomask"
         expected = "test = node.session.auth.password -v *** nomask"
         self.assertEqual(expected, strutils.mask_password(payload))
 
-        payload = ("test = node.session.auth.password --password mypassword "
+        payload = ("test = node.session.auth.password --password TL0EfN33 "
                    "nomask")
         expected = ("test = node.session.auth.password --password *** "
                     "nomask")
         self.assertEqual(expected, strutils.mask_password(payload))
 
-        payload = ("test = node.session.auth.password --password mypassword")
+        payload = ("test = node.session.auth.password --password TL0EfN33")
         expected = ("test = node.session.auth.password --password ***")
         self.assertEqual(expected, strutils.mask_password(payload))
 
@@ -568,6 +613,83 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
         self.assertEqual(expected, strutils.mask_password(payload))
 
 
+class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
+
+    def test_dictionary(self):
+        payload = {'password': 'TL0EfN33'}
+        expected = {'password': '***'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+        payload = {'user': 'admin', 'password': 'TL0EfN33'}
+        expected = {'user': 'admin', 'password': '***'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+        payload = {'strval': 'somestring',
+                   'dictval': {'user': 'admin', 'password': 'TL0EfN33'}}
+        expected = {'strval': 'somestring',
+                    'dictval': {'user': 'admin', 'password': '***'}}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+        payload = {'strval': '--password abc',
+                   'dont_change': 'this is fine',
+                   'dictval': {'user': 'admin', 'password': b'TL0EfN33'}}
+        expected = {'strval': '--password ***',
+                    'dont_change': 'this is fine',
+                    'dictval': {'user': 'admin', 'password': '***'}}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+        payload = {'ipmi_password': 'KeDrahishvowphyecMornEm0or('}
+        expected = {'ipmi_password': '***'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+    def test_do_no_harm(self):
+        payload = {}
+        expected = {}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+        payload = {'somekey': 'somevalue',
+                   'anotherkey': 'anothervalue'}
+        expected = {'somekey': 'somevalue',
+                    'anotherkey': 'anothervalue'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+    def test_mask_values(self):
+        payload = {'somekey': 'test = cmd --password my\xe9\x80\x80pass'}
+        expected = {'somekey': 'test = cmd --password ***'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+    def test_other_non_str_values(self):
+        payload = {'password': 'DK0PK1AK3', 'bool': True,
+                   'dict': {'cat': 'meow', 'password': "*aa38skdjf"},
+                   'float': 0.1, 'int': 123, 'list': [1, 2], 'none': None,
+                   'str': 'foo'}
+        expected = {'password': '***', 'bool': True,
+                    'dict': {'cat': 'meow', 'password': '***'},
+                    'float': 0.1, 'int': 123, 'list': [1, 2], 'none': None,
+                    'str': 'foo'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+    def test_argument_untouched(self):
+        """Make sure that the argument passed in is not modified"""
+        payload = {'password': 'DK0PK1AK3', 'bool': True,
+                   'dict': {'cat': 'meow', 'password': "*aa38skdjf"},
+                   'float': 0.1, 'int': 123, 'list': [1, 2], 'none': None,
+                   'str': 'foo'}
+        pristine = copy.deepcopy(payload)
+        # Send the payload into the function, to see if it gets modified
+        strutils.mask_dict_password(payload)
+        self.assertEqual(pristine, payload)
+
+
 class IsIntLikeTestCase(test_base.BaseTestCase):
     def test_is_int_like_true(self):
         self.assertTrue(strutils.is_int_like(1))
@@ -592,3 +714,135 @@ class IsIntLikeTestCase(test_base.BaseTestCase):
         # NOTE(viktors): Check integer numbers with base not 10
         self.assertFalse(strutils.is_int_like("0o51"))
         self.assertFalse(strutils.is_int_like("0xDEADBEEF"))
+
+
+class StringLengthTestCase(test_base.BaseTestCase):
+    def test_check_string_length(self):
+        self.assertIsNone(strutils.check_string_length(
+                          'test', 'name', max_length=255))
+        self.assertRaises(ValueError,
+                          strutils.check_string_length,
+                          '', 'name', min_length=1)
+        self.assertRaises(ValueError,
+                          strutils.check_string_length,
+                          'a' * 256, 'name', max_length=255)
+        self.assertRaises(TypeError,
+                          strutils.check_string_length,
+                          11, 'name', max_length=255)
+        self.assertRaises(TypeError,
+                          strutils.check_string_length,
+                          dict(), 'name', max_length=255)
+
+    def test_check_string_length_noname(self):
+        self.assertIsNone(strutils.check_string_length(
+                          'test', max_length=255))
+        self.assertRaises(ValueError,
+                          strutils.check_string_length,
+                          '', min_length=1)
+        self.assertRaises(ValueError,
+                          strutils.check_string_length,
+                          'a' * 256, max_length=255)
+        self.assertRaises(TypeError,
+                          strutils.check_string_length,
+                          11, max_length=255)
+        self.assertRaises(TypeError,
+                          strutils.check_string_length,
+                          dict(), max_length=255)
+
+
+class SplitPathTestCase(test_base.BaseTestCase):
+    def test_split_path_failed(self):
+        self.assertRaises(ValueError, strutils.split_path, '')
+        self.assertRaises(ValueError, strutils.split_path, '/')
+        self.assertRaises(ValueError, strutils.split_path, '//')
+        self.assertRaises(ValueError, strutils.split_path, '//a')
+        self.assertRaises(ValueError, strutils.split_path, '/a/c')
+        self.assertRaises(ValueError, strutils.split_path, '//c')
+        self.assertRaises(ValueError, strutils.split_path, '/a/c/')
+        self.assertRaises(ValueError, strutils.split_path, '/a//')
+        self.assertRaises(ValueError, strutils.split_path, '/a', 2)
+        self.assertRaises(ValueError, strutils.split_path, '/a', 2, 3)
+        self.assertRaises(ValueError, strutils.split_path, '/a', 2, 3, True)
+        self.assertRaises(ValueError, strutils.split_path, '/a/c/o/r', 3, 3)
+        self.assertRaises(ValueError, strutils.split_path, '/a', 5, 4)
+
+    def test_split_path_success(self):
+        self.assertEqual(strutils.split_path('/a'), ['a'])
+        self.assertEqual(strutils.split_path('/a/'), ['a'])
+        self.assertEqual(strutils.split_path('/a/c', 2), ['a', 'c'])
+        self.assertEqual(strutils.split_path('/a/c/o', 3), ['a', 'c', 'o'])
+        self.assertEqual(strutils.split_path('/a/c/o/r', 3, 3, True),
+                         ['a', 'c', 'o/r'])
+        self.assertEqual(strutils.split_path('/a/c', 2, 3, True),
+                         ['a', 'c', None])
+        self.assertEqual(strutils.split_path('/a/c/', 2), ['a', 'c'])
+        self.assertEqual(strutils.split_path('/a/c/', 2, 3), ['a', 'c', ''])
+
+    def test_split_path_invalid_path(self):
+        try:
+            strutils.split_path('o\nn e', 2)
+        except ValueError as err:
+            self.assertEqual(str(err), 'Invalid path: o%0An%20e')
+        try:
+            strutils.split_path('o\nn e', 2, 3, True)
+        except ValueError as err:
+            self.assertEqual(str(err), 'Invalid path: o%0An%20e')
+
+
+class SplitByCommas(test_base.BaseTestCase):
+    def test_not_closed_quotes(self):
+        self.assertRaises(ValueError, strutils.split_by_commas, '"ab","b""')
+
+    def test_no_comma_before_opening_quotes(self):
+        self.assertRaises(ValueError, strutils.split_by_commas, '"ab""b"')
+
+    def test_quote_inside_unquoted(self):
+        self.assertRaises(ValueError, strutils.split_by_commas, 'a"b,cd')
+
+    def check(self, expect, input):
+        self.assertEqual(expect, strutils.split_by_commas(input))
+
+    def test_plain(self):
+        self.check(["a,b", "ac"], '"a,b",ac')
+
+    def test_with_backslash_inside_quoted(self):
+        self.check(['abc"', 'de', 'fg,h', 'klm\\', '"nop'],
+                   r'"abc\"","de","fg,h","klm\\","\"nop"')
+
+    def test_with_backslash_inside_unquoted(self):
+        self.check([r'a\bc', 'de'], r'a\bc,de')
+
+    def test_with_escaped_quotes_in_row_inside_quoted(self):
+        self.check(['a"b""c', 'd'], r'"a\"b\"\"c",d')
+
+
+@ddt.ddt
+class ValidateIntegerTestCase(test_base.BaseTestCase):
+
+    @ddt.unpack
+    @ddt.data({"value": 42, "name": "answer", "output": 42},
+              {"value": "42", "name": "answer", "output": 42},
+              {"value": "7", "name": "lucky", "output": 7,
+               "min_value": 7, "max_value": 8},
+              {"value": 7, "name": "lucky", "output": 7,
+               "min_value": 6, "max_value": 7},
+              {"value": 300, "name": "Spartaaa!!!", "output": 300,
+               "min_value": 300},
+              {"value": "300", "name": "Spartaaa!!!", "output": 300,
+               "max_value": 300})
+    def test_valid_inputs(self, output, value, name, **kwargs):
+        self.assertEqual(strutils.validate_integer(value, name,
+                                                   **kwargs), output)
+
+    @ddt.unpack
+    @ddt.data({"value": "im-not-an-int", "name": ''},
+              {"value": 3.14, "name": "Pie"},
+              {"value": "299", "name": "Sparta no-show",
+               "min_value": 300, "max_value": 300},
+              {"value": 55, "name": "doing 55 in a 54",
+               "max_value": 54},
+              {"value": six.unichr(129), "name": "UnicodeError",
+               "max_value": 1000})
+    def test_invalid_inputs(self, value, name, **kwargs):
+        self.assertRaises(ValueError, strutils.validate_integer,
+                          value, name, **kwargs)
