@@ -15,6 +15,7 @@
 
 import calendar
 import datetime
+import logging
 import time
 
 import iso8601
@@ -38,8 +39,8 @@ class TimeUtilsTest(test_base.BaseTestCase):
         self.skynet_self_aware_time_str = '1997-08-29T06:14:00Z'
         self.skynet_self_aware_time_ms_str = '1997-08-29T06:14:00.000123Z'
         self.skynet_self_aware_time = datetime.datetime(1997, 8, 29, 6, 14, 0)
-        self.skynet_self_aware_ms_time = datetime.datetime(
-            1997, 8, 29, 6, 14, 0, 123)
+        self.skynet_self_aware_ms_time = datetime.datetime(1997, 8, 29, 6, 14,
+                                                           0, 123)
         self.one_minute_before = datetime.datetime(1997, 8, 29, 6, 13, 0)
         self.one_minute_after = datetime.datetime(1997, 8, 29, 6, 15, 0)
         self.skynet_self_aware_time_perfect_str = '1997-08-29T06:14:00.000000'
@@ -112,6 +113,11 @@ class TimeUtilsTest(test_base.BaseTestCase):
         self._test_is_older_than(lambda x: x.replace(
             tzinfo=iso8601.iso8601.UTC))
 
+    def test_is_older_than_aware_no_utc(self):
+        self._test_is_older_than(lambda x: x.replace(
+            tzinfo=iso8601.iso8601.FixedOffset(1, 0, 'foo')).replace(
+                hour=7))
+
     def _test_is_newer_than(self, fn):
         strptime = datetime.datetime.strptime
         with mock.patch('datetime.datetime') as datetime_mock:
@@ -138,6 +144,11 @@ class TimeUtilsTest(test_base.BaseTestCase):
         self._test_is_newer_than(lambda x: x.replace(
             tzinfo=iso8601.iso8601.UTC))
 
+    def test_is_newer_than_aware_no_utc(self):
+        self._test_is_newer_than(lambda x: x.replace(
+            tzinfo=iso8601.iso8601.FixedOffset(1, 0, 'foo')).replace(
+                hour=7))
+
     def test_set_time_override_using_default(self):
         now = timeutils.utcnow_ts()
 
@@ -191,6 +202,28 @@ class TimeUtilsTest(test_base.BaseTestCase):
         backagain = timeutils.unmarshall_time(binary)
         self.assertEqual(now, backagain)
 
+    def test_marshall_time_with_tz(self):
+        now = timeutils.utcnow()
+        now = now.replace(tzinfo=iso8601.iso8601.UTC)
+        binary = timeutils.marshall_now(now)
+        self.assertEqual("UTC", binary['tzname'])
+        backagain = timeutils.unmarshall_time(binary)
+        self.assertEqual(now, backagain)
+        self.assertIsNotNone(backagain.tzinfo)
+        self.assertEqual(now.utcoffset(), backagain.utcoffset())
+
+    def test_unmarshall_time_leap_second(self):
+        leap_dict = dict(day=30, month=6, year=2015,
+                         hour=23, minute=59,
+                         second=timeutils._MAX_DATETIME_SEC + 1,
+                         microsecond=0)
+        leap_time = timeutils.unmarshall_time(leap_dict)
+
+        leap_dict.update(second=timeutils._MAX_DATETIME_SEC)
+        expected = timeutils.unmarshall_time(leap_dict)
+
+        self.assertEqual(expected, leap_time)
+
     def test_delta_seconds(self):
         before = timeutils.utcnow()
         after = before + datetime.timedelta(days=7, seconds=59,
@@ -198,11 +231,6 @@ class TimeUtilsTest(test_base.BaseTestCase):
         self.assertAlmostEquals(604859.123456,
                                 timeutils.delta_seconds(before, after))
 
-    def test_total_seconds(self):
-        delta = datetime.timedelta(days=1, hours=2, minutes=3, seconds=4.5)
-        self.assertAlmostEquals(93784.5,
-                                timeutils.total_seconds(delta))
-
     def test_iso8601_from_timestamp(self):
         utcnow = timeutils.utcnow()
         iso = timeutils.isotime(utcnow)
@@ -352,6 +380,99 @@ class TestIso8601Time(test_base.BaseTestCase):
         self.assertTrue(naive < dt)
 
 
+class TimeItTest(test_base.BaseTestCase):
+
+    @mock.patch('time.sleep')
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_timed(self, mock_now, mock_sleep):
+        mock_now.side_effect = monotonic_iter(incr=0.1)
+        fake_logger = mock.MagicMock(logging.getLogger(), autospec=True)
+
+        @timeutils.time_it(fake_logger)
+        def slow_function():
+            time.sleep(0.1)
+
+        slow_function()
+        self.assertTrue(mock_now.called)
+        self.assertTrue(mock_sleep.called)
+        self.assertTrue(fake_logger.log.called)
+        fake_logger.log.assert_called_with(logging.DEBUG, mock.ANY, mock.ANY)
+
+    @mock.patch('time.sleep')
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_no_timed_disabled(self, mock_now, mock_sleep):
+        mock_now.side_effect = monotonic_iter(incr=0.1)
+        fake_logger = mock.MagicMock(logging.getLogger(), autospec=True)
+
+        @timeutils.time_it(fake_logger, enabled=False)
+        def slow_function():
+            time.sleep(0.1)
+
+        slow_function()
+        self.assertFalse(mock_now.called)
+        self.assertFalse(fake_logger.log.called)
+
+    @mock.patch('time.sleep')
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_no_timed_to_fast(self, mock_now, mock_sleep):
+        mock_now.side_effect = monotonic_iter(incr=0.1)
+        fake_logger = mock.MagicMock(logging.getLogger(), autospec=True)
+
+        @timeutils.time_it(fake_logger, min_duration=10)
+        def fast_function():
+            pass
+
+        fast_function()
+        self.assertFalse(fake_logger.log.called)
+
+    @mock.patch('time.sleep')
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_no_timed_exception(self, mock_now, mock_sleep):
+        mock_now.side_effect = monotonic_iter(incr=0.1)
+        fake_logger = mock.MagicMock(logging.getLogger(), autospec=True)
+
+        @timeutils.time_it(fake_logger)
+        def broken_function():
+            raise IOError("Broken")
+
+        self.assertRaises(IOError, broken_function)
+        self.assertFalse(fake_logger.log.called)
+
+    @mock.patch('time.sleep')
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_timed_custom_message(self, mock_now, mock_sleep):
+        mock_now.side_effect = monotonic_iter(incr=0.1)
+        fake_logger = mock.MagicMock(logging.getLogger(), autospec=True)
+
+        @timeutils.time_it(fake_logger, message="That took a long time")
+        def slow_function():
+            time.sleep(0.1)
+
+        slow_function()
+        self.assertTrue(mock_now.called)
+        self.assertTrue(mock_sleep.called)
+        self.assertTrue(fake_logger.log.called)
+        fake_logger.log.assert_called_with(logging.DEBUG,
+                                           "That took a long time",
+                                           mock.ANY)
+
+    @mock.patch('time.sleep')
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_timed_custom_level(self, mock_now, mock_sleep):
+        mock_now.side_effect = monotonic_iter(incr=0.1)
+        fake_logger = mock.MagicMock(logging.getLogger(), autospec=True)
+
+        @timeutils.time_it(fake_logger, log_level=logging.INFO)
+        def slow_function():
+            time.sleep(0.1)
+
+        slow_function()
+        self.assertTrue(mock_now.called)
+        self.assertTrue(mock_sleep.called)
+        self.assertTrue(fake_logger.log.called)
+        fake_logger.log.assert_called_with(logging.INFO, mock.ANY, mock.ANY)
+
+
 class StopWatchTest(test_base.BaseTestCase):
     def test_leftover_no_duration(self):
         watch = timeutils.StopWatch()
